/////////////////////////////////////////////////////////////////////////////
// GliD3D_Sub.cpp : No comment
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Directive

#include "StdHdr.h"
#pragma hdrstop
#include "GliD3D.h"
#include "GliD3D_Sub.h"

/////////////////////////////////////////////////////////////////////////////
// Defines / Variables

/////////////////////////////////////////////////////////////////////////////
// Prototypes

/////////////////////////////////////////////////////////////////////////////
// Implements

DWORD
grsubConvColor(GrColor_t color)
{
	// Convert color format to ARGB 
	switch (g_gr_state.color_format) {
	case GR_COLORFORMAT_RGBA:
		return (DWORD)(
			   ((((DWORD)color) >> 8) & 0x00ffffff) | ((((DWORD)color) << 24) & 0xff000000));
	case GR_COLORFORMAT_ARGB:
		return (DWORD)color;
	case GR_COLORFORMAT_BGRA:
		return (DWORD)(
			   ((((DWORD)color) << 8) & 0x00ff0000) | ((((DWORD)color) >> 8) & 0x0000ff00) |
			   ((((DWORD)color) >> 24) & 0x000000ff) | ((((DWORD)color) << 24) & 0xff000000));
	case GR_COLORFORMAT_ABGR:
		return (DWORD)(
			   ((((DWORD)color) << 16) & 0x00ff0000) | ((((DWORD)color)) & 0xff00ff00) |
			   ((((DWORD)color) >> 16) & 0x000000ff));
	}
	return 0;
}

DWORD
grsubConvColorRef(GrColor_t color)
{
	// Convert color format to COLORREF (BGR) 
	switch (g_gr_state.color_format) {
	case GR_COLORFORMAT_RGBA:
		return (COLORREF)(
			   ((((DWORD)color) >> 24) & 0x000000ff) | ((((DWORD)color) <<  8) & 0x0000ff00) |
			   ((((DWORD)color) <<  8) & 0x00ff0000));
	case GR_COLORFORMAT_ARGB:
		return (COLORREF)(
			   ((((DWORD)color) >> 16) & 0x000000ff) | ((((DWORD)color) << 16) & 0x00ff0000) |
			   ((((DWORD)color)) & 0x0000ff00));
	case GR_COLORFORMAT_BGRA:
		return (COLORREF)((((DWORD)color) >> 8) & 0x00ffffff);
	case GR_COLORFORMAT_ABGR:
		return (COLORREF)color & 0x00ffffff;
	}
	return 0;
}

VOID
grsubInitRenderState()
{
	g_gr_pd3ddevice->SetLightState(D3DLIGHTSTATE_MATERIAL, NULL);
}

VOID
grsubPreRender()
{
//	GB3DBeginRender();
	
//	if (0)
	if ((g_gr_state.color_combine_other != GR_COMBINE_OTHER_TEXTURE))
		g_gr_pd3ddevice->SetTexture(0, NULL);
	else {
//		TRACE("GliD3D info : With Texture %d(%08x)\n", g_gr_tmu[0].texture_source_start_address, g_gr_tmu[0].texture[g_gr_tmu[0].texture_source_start_address].pd3dtexture);
		GLID3D_TMU* ptmu = &g_gr_tmu[0];
		if (ptmu->texture_source_start_address_old != ptmu->texture_source_start_address) {
			g_gr_pd3ddevice->SetTexture(0, ptmu->texture[ptmu->texture_source_start_address].pd3dtexture);
//			g_gr_pd3ddevice->SetTexture(0, ptmu->texture[0].pd3dtexture);
//			g_gr_pd3ddevice->SetTexture(0, NULL);
			ptmu->texture_source_start_address_old = ptmu->texture_source_start_address;
			if (ptmu->texture[ptmu->texture_source_start_address].pd3dtexture && g_gr_state.chromakey_mode == GR_CHROMAKEY_ENABLE) {
				DDCOLORKEY ddck;
				ddck.dwColorSpaceLowValue =
				ddck.dwColorSpaceHighValue = g_gr_state.chromakey_value;
				ptmu->texture[ptmu->texture_source_start_address].pddsurface->SetColorKey(DDCKEY_SRCBLT, &ddck);
			}
		}
	}
/*g_gr_pd3ddevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTFN_POINT);
g_gr_pd3ddevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTFN_POINT);
g_gr_pd3ddevice->SetRenderState(D3DRENDERSTATE_TEXTUREMIN, D3DFILTER_NEAREST);
g_gr_pd3ddevice->SetRenderState(D3DRENDERSTATE_TEXTUREMAG, D3DFILTER_NEAREST);
*/
}

VOID
grsubPostRender()
{
//	GB3DEndRender();
}

INT
grsubGetTmuIndex(GrChipID_t tmu)
{
	switch (tmu) {
	case GR_TMU0: return GLID3D_TMU0;
	}
	return GLID3D_TMU0;
}

BOOL
gusub3dfGetInfo(LPCSTR filename, Gu3dfInfo *info, LPDWORD pdwDataOffset)
{
	CHAR pBuf[256];
	CHAR* pnt;


	// Open file
	HANDLE hFile;
	DWORD dwLengthLo, dwLengthHi, dwBytesRead;
	INT nSize = 0;
	if ((hFile = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ, NULL, 
		OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL)) == INVALID_HANDLE_VALUE) {
		return FALSE;
	}
	dwLengthLo = GetFileSize(hFile, &dwLengthHi);
	

	// Valivate Header strings
	ZeroMemory(pBuf, sizeof(pBuf));
	ReadFile(hFile, pBuf, 9, &dwBytesRead, NULL);
	pBuf[8] = '\x0';
	TRACE("------------------------------------------------------------\n");
	TRACE("GliD3D info : Filename = %s\n", filename);
	TRACE("GliD3D info : 3DF Header = %s\n", pBuf);
	if (strnicmp(pBuf, "3df v", 5) != 0) {
		CloseHandle(hFile);
		return FALSE;
	}
	nSize += 9;


	// Read FORMAT
	pnt = pBuf;
	for (;;) {
		ReadFile(hFile, pnt, 1, &dwBytesRead, NULL);
		if (*pnt == '\x0a') {
			*pnt = '\x00';
			nSize++;
			break;
		}
		pnt++;
		nSize++;
	}
	TRACE("                  Format = %s\n", pBuf);

	GrTextureFormat_t format;

	if (stricmp(pBuf, "yiq422") == 0) {
		// Not supported yet
		//format = GR_TEXFMT_YIQ_422;
		format = -1;
	} else if (stricmp(pBuf, "p8") == 0) {
		// Not supported yet
		format = GR_TEXFMT_P_8;
	} else if (stricmp(pBuf, "alpha8") == 0) {
		// Not supported yet
		//format = GR_TEXFMT_ALPHA_8;
		format = -1;
	} else if (stricmp(pBuf, "i8") == 0) {
		// Not supported yet
		//format = GR_TEXFMT_INTENSITY_8;
		format = -1;
	} else if (stricmp(pBuf, "ai44") == 0) {
		// Not supported yet
		//format = GR_TEXFMT_ALPHA_INTENSITY_44;
		format = -1;
	} else if (stricmp(pBuf, "rgb565") == 0) {
		// Not supported yet
		format = GR_TEXFMT_RGB_565;
	} else if (stricmp(pBuf, "argb8332") == 0) {
		// Not supported yet
		//format = GR_TEXFMT_ARGB_8332;
		format = -1;
	} else if (stricmp(pBuf, "argb1555") == 0) {
		// Not supported yet
		format = GR_TEXFMT_ARGB_1555;
	} else if (stricmp(pBuf, "argb4444") == 0) {
		// Not supported yet
		format = GR_TEXFMT_ARGB_4444;
	} else if (stricmp(pBuf, "ayiq8422") == 0) {
		// Not supported yet
		//format = GR_TEXFMT_AYIQ_8422;
		format = -1;
	} else if (stricmp(pBuf, "ai88") == 0) {
		// Not supported yet
		//format = GR_TEXFMT_ALPHA_INTENSITY_88;
		format = -1;
	}
	if (format == -1) {
		CloseHandle(hFile);
		return FALSE;
	}


	// Read LOD RANGE
	pnt = pBuf;
	for (;;) {
		ReadFile(hFile, pnt, 1, &dwBytesRead, NULL);
		if (*pnt == '\x0a') {
			*pnt = '\x00';
			nSize++;
			break;
		}
		pnt++;
		nSize++;
	}
	TRACE("                  Lod range = %s\n", pBuf);
	if (strnicmp(pBuf, "lod range:", 10) != 0) {
		CloseHandle(hFile);
		return FALSE;
	}

	INT nSmallLod, nLargeLod;
	GrLOD_t small_lod, large_lod;

	sscanf(pBuf + 10, "%d %d", &nSmallLod, &nLargeLod);

	small_lod = grsubIntToLod(nSmallLod);
	large_lod = grsubIntToLod(nLargeLod);


	// Read ASPECT RATIO
	pnt = pBuf;
	for (;;) {
		ReadFile(hFile, pnt, 1, &dwBytesRead, NULL);
		if (*pnt == '\x0a') {
			*pnt = '\x00';
			nSize++;
			break;
		}
		pnt++;
		nSize++;
	}
	TRACE("                  Aspect ratio = %s\n", pBuf);
	if (strnicmp(pBuf, "aspect ratio:", 13) != 0) {
		CloseHandle(hFile);
		return FALSE;
	}

	INT nAspectW, nAspectH;
	GrAspectRatio_t aspect_ratio;

	sscanf(pBuf + 13, "%d %d", &nAspectW, &nAspectH);

	grsubIntToAspect(&aspect_ratio, nAspectW, nAspectH);


	// Close file
	CloseHandle(hFile);
	
	// Set parameter
	info->header.width = nLargeLod * nAspectW;
	info->header.height = nLargeLod * nAspectH;
	info->header.small_lod = small_lod;
	info->header.large_lod = large_lod;
	info->header.aspect_ratio = aspect_ratio;
	info->header.format = format;
	info->mem_required = dwLengthLo - nSize;

	TRACE("                  Mem required = %d Bytes\n", info->mem_required);
	TRACE("------------------------------------------------------------\n\n");

	if (pdwDataOffset) {
		*pdwDataOffset = nSize;
	}

	return TRUE;
}

VOID
grsubGetSurfacePixelFormat(LPGLID3D_PIXELFORMAT ppixel, LPDIRECTDRAWSURFACE4 pddsurface)
{
	INT i;
	DDSURFACEDESC2 ddsd;
	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	pddsurface->GetSurfaceDesc(&ddsd);

	if (ddsd.ddpfPixelFormat.dwFlags & DDPF_PALETTEINDEXED8) {
		ppixel->bPalette = TRUE;
		ppixel->nBits = 8;
	} else {

		ppixel->bPalette = FALSE;
		ppixel->nBits = ddsd.ddpfPixelFormat.dwRGBBitCount;

		INT nShifts, nBits;
		DWORD dwMask;
		nShifts = 0;
		nBits = 0;
		dwMask = ddsd.ddpfPixelFormat.dwRBitMask;
		for (i = 0; i < 32; i++) {
			if (dwMask & 1) break;
			dwMask >>= 1;
			nShifts++;
		}
		dwMask = ddsd.ddpfPixelFormat.dwRBitMask;
		for (i = 0; i < 32; i++) {
			if (dwMask & 1) nBits++;
			dwMask >>= 1;
		}
		ppixel->nRShifts = nShifts;
		ppixel->nRMax = (1 << nBits) - 1;
		nShifts = 0;
		nBits = 0;
		dwMask = ddsd.ddpfPixelFormat.dwGBitMask;
		for (i = 0; i < 32; i++) {
			if (dwMask & 1) break;
			dwMask >>= 1;
			nShifts++;
		}
		dwMask = ddsd.ddpfPixelFormat.dwGBitMask;
		for (i = 0; i < 32; i++) {
			if (dwMask & 1) nBits++;
			dwMask >>= 1;
		}
		ppixel->nGShifts = nShifts;
		ppixel->nGMax = (1 << nBits) - 1;
		nShifts = 0;
		nBits = 0;
		dwMask = ddsd.ddpfPixelFormat.dwBBitMask;
		for (i = 0; i < 32; i++) {
			if (dwMask & 1) break;
			dwMask >>= 1;
			nShifts++;
		}
		dwMask = ddsd.ddpfPixelFormat.dwBBitMask;
		for (i = 0; i < 32; i++) {
			if (dwMask & 1) nBits++;
			dwMask >>= 1;
		}
		ppixel->nBShifts = nShifts;
		ppixel->nBMax = (1 << nBits) - 1;
/*		TRACE("GliD3D info : Surface pixel format : R%d(%d) G%d(%d) B%d(%d)\n",
			ppixel->nRShifts, ppixel->nRMax,
			ppixel->nGShifts, ppixel->nGMax,
			ppixel->nBShifts, ppixel->nBMax
		);
*/	}
}

INT
grsubLodToInt(GrLOD_t lod)
{
	switch (lod) {
	case GR_LOD_LOG2_1  : return 1;
	case GR_LOD_LOG2_2  : return 2;
	case GR_LOD_LOG2_4  : return 4;
	case GR_LOD_LOG2_8  : return 8;
	case GR_LOD_LOG2_16 : return 16;
	case GR_LOD_LOG2_32 : return 32;
	case GR_LOD_LOG2_64 : return 64;
	case GR_LOD_LOG2_128: return 128;
	case GR_LOD_LOG2_256: return 256;
	}
	return 0;
}

GrLOD_t
grsubIntToLod(INT nLod)
{
	switch (nLod) {
	case 1   : return GR_LOD_LOG2_1;
	case 2   : return GR_LOD_LOG2_2;
	case 4   : return GR_LOD_LOG2_4;
	case 8   : return GR_LOD_LOG2_8;
	case 16  : return GR_LOD_LOG2_16;
	case 32  : return GR_LOD_LOG2_32;
	case 64  : return GR_LOD_LOG2_64;
	case 128 : return GR_LOD_LOG2_128;
	case 256 : return GR_LOD_LOG2_256;
	}
	return 0;
}

VOID
grsubAspectToInt(LPINT pnAspectW, LPINT pnAspectH, GrAspectRatio_t aspect_ratio)
{
	switch (aspect_ratio) {
	case GR_ASPECT_LOG2_8x1: *pnAspectW = 8; *pnAspectH = 1; return;
	case GR_ASPECT_LOG2_4x1: *pnAspectW = 4; *pnAspectH = 1; return;
	case GR_ASPECT_LOG2_2x1: *pnAspectW = 2; *pnAspectH = 1; return;
	case GR_ASPECT_LOG2_1x1: *pnAspectW = 1; *pnAspectH = 1; return;
	case GR_ASPECT_LOG2_1x2: *pnAspectW = 1; *pnAspectH = 2; return;
	case GR_ASPECT_LOG2_1x4: *pnAspectW = 1; *pnAspectH = 4; return;
	case GR_ASPECT_LOG2_1x8: *pnAspectW = 1; *pnAspectH = 8; return;
	}
}

VOID
grsubIntToAspect(GrAspectRatio_t* paspect_ratio, INT nAspectW, INT nAspectH)
{
	if (nAspectW == 1) {
		if (nAspectH == 1) *paspect_ratio = GR_ASPECT_LOG2_1x1; return;
		if (nAspectH == 2) *paspect_ratio = GR_ASPECT_LOG2_1x2; return;
		if (nAspectH == 4) *paspect_ratio = GR_ASPECT_LOG2_1x4; return;
		if (nAspectH == 8) *paspect_ratio = GR_ASPECT_LOG2_1x8; return;
	} else {
		if (nAspectW == 2) *paspect_ratio = GR_ASPECT_LOG2_2x1; return;
		if (nAspectW == 4) *paspect_ratio = GR_ASPECT_LOG2_4x1; return;
		if (nAspectW == 8) *paspect_ratio = GR_ASPECT_LOG2_8x1; return;
	}
}

/////////////////////////////////////////////////////////////////////////////
